import * as auth from "$lib/server/auth";
import { forbiddenResponse, unauthorizedResponse } from "$lib/server/responseBodies";
import { routeAuth, type Method } from "$lib/server/routeAuth";
import { type Handle } from "@sveltejs/kit";

export const handle: Handle = async ({ event, resolve }) => {
	const creds = await auth.authenticate(event);

	if (!creds) {
		return unauthorizedResponse();
	}

	const authResult = auth.isAuthorized(
		routeAuth,
		event.request.method as Method,
		event.url.pathname,
		creds,
	);

	if (authResult === auth.AuthorizationResult.Denied) {
		return forbiddenResponse();
	} else if (authResult === auth.AuthorizationResult.Unauthenticated) {
		return unauthorizedResponse();
	}

	event.locals.user = creds;

	return await resolve(event);
};
