import type { Listing } from "$lib/Listing";
import { MongoClient, ObjectId, ServerApiVersion, type Document, type WithId } from "mongodb";
import type { Id } from "../Id";

type ListingFromMongo = Omit<Listing, "id"> & { _id: ObjectId };

const uri = `mongodb://127.0.0.1:27017`;
const DATABASE = "ten-thousand";
let client: MongoClient | null = null;

async function init() {
	const c = new MongoClient(uri, {
		serverApi: {
			version: ServerApiVersion.v1,
			strict: true,
			deprecationErrors: true
		}
	});

	await c.connect();
	return c;
}

export async function writeListing(col: string, listing: Listing) {
	if (client === null) {
		client = await init();
	}

	await client.db(DATABASE).collection(col).insertOne(fixListingForMongo(listing));
}

export async function readListingById<T>(
	col: string,
	id: Id,
	dataGuard: (target: unknown) => target is T
) {}

export async function readListingByQuery<T>(
	col: string,
	query: object,
	dataGuard: (target: unknown) => target is Listing<T>
): Promise<Listing<T> | null> {
	if (client === null) {
		client = await init();
	}

	const res = await client.db(DATABASE).collection(col).findOne(query);

	if (res === null) {
		return null;
	}

	return fixListingFromMongo(res, dataGuard);
}

function fixListingForMongo(listing: Listing): ListingFromMongo {
	// TODO: These two statements are tricky without any. Perhaps id could be optional,
	//       but that's kind of stupid because it's not optional on the type I'm
	//       constructing. Figure out something better here.
	const adjusted: any = { _id: listing.id, ...listing };
	delete adjusted.id;
	return adjusted;
}

function fixListingFromMongo<T>(
	target: WithId<Document>,
	dataGuard: (target: unknown) => target is Listing<T>
): Listing<T> {
	// TODO: These two statements are tricky without any. Perhaps id could be optional,
	//       but that's kind of stupid because it's not optional on the type I'm
	//       constructing. Figure out something better here.
	const adjusted: any = { id: target._id, ...target };
	delete adjusted._id;

	if (!dataGuard(adjusted)) {
		throw new Error("the returned document does not conform to the provided type");
	}

	return adjusted;
}
